#
# Transfer interface type to target-side code

getsym $coff _iftype
PutLong $DEV $_iftype_addr $_iftype_pg $iftype

#
# Transfer controller base address to target-side code
#
getsym $coff _ctrlr_base
PutLong $DEV $_ctrlr_base_addr $_ctrlr_base_pg $ctrlr_base

#
# CS pins autoplacement
# if user specifies "set cs_port_address auto" 
# then define CS, connected to controller's dedicated pin
#

if { [string equal -nocase $cs_port_address "auto"] } {

  if { [string equal $cpu_type "TMS470R1B"] } {
    # cs resides in SPIPC3 register
    set cs_port_address [expr $ctrlr_base + 0x24]
    set cs_port_mempage 0
    # register is 32-bit wide
    set cs_port_width 2
    # cs is controlled by bit 4 (0x10)
    set cs_port_bit 0x10
    # cs polarity is active low
    set cs_port_polarity 0
  }

}

#
# Transfer CS pin info to target-side code
#
getsym $coff _cs_port_address
getsym $coff _cs_port_mempage
getsym $coff _cs_port_width
getsym $coff _cs_port_bit
getsym $coff _cs_port_polarity

PutLong $DEV $_cs_port_address_addr $_cs_port_address_pg $cs_port_address
PutLong $DEV $_cs_port_mempage_addr $_cs_port_mempage_pg $cs_port_mempage
PutLong $DEV $_cs_port_width_addr $_cs_port_width_pg $cs_port_width
PutLong $DEV $_cs_port_bit_addr $_cs_port_bit_pg $cs_port_bit
PutLong $DEV $_cs_port_polarity_addr $_cs_port_polarity_pg $cs_port_polarity

#
# Initialize SPI controller
#

if { [string equal $cpu_type "TMS470R1B"] } {

  puts "Initializing SPI controller (@$ctrlr_base)..." 

  #
  # Calculate divide factor for ~ 1 MHz SCK
  #

  set divisor [expr $periph_clock / 1000000]
  puts "Calculated CLKDIV value is $divisor"

  # SPICTRL2 = 0x00000000
  PutLong $DEV [expr $ctrlr_base + 0x04] 0 0x00000000
  # pause
  after 10

  # SPICTRL1; divisor and bit count
  PutLong $DEV [expr $ctrlr_base + 0x00] 0 [expr ($divisor << 5) | 8]

  # SPICTRL2; PHASE=1 CLKMOD=1 MASTER=1
  PutLong $DEV [expr $ctrlr_base + 0x04] 0 0x00000029

  # SPICTRL3; 0
  PutLong $DEV [expr $ctrlr_base + 0x08] 0 0x00000000

  # pins: CS and ENA - as GPIO
  # SPIPC6;
  PutLong $DEV [expr $ctrlr_base + 0x30] 0 0x0000000E

  # directions - CS as output
  # SPIPC1;
  PutLong $DEV [expr $ctrlr_base + 0x1C] 0 0x00000010
  
  # value - CS = 1
  # SPIPC3;
  PutLong $DEV [expr $ctrlr_base + 0x24] 0 0x00000010

  # SPICTRL2; out of reset
  PutLong $DEV [expr $ctrlr_base + 0x04] 0 0x00000039

  # dummy read from SPIBUF to clear overrun and rdy flags
  ReadLong $DEV [expr $ctrlr_base + 0x14] 0

}

if { [string equal $cpu_type "TMS320C2800"] } {

  set mempage 1
  puts "Initializing SPI controller ($ctrlr_base)..." 

  #
  # Calculate divide factor for ~ 1 MHz SCK
  #

  set divisor [expr $periph_clock / 1000000-1]

  puts "Calculated CLKDIV value is $divisor"
  

  if { [string equal $cpu_type "TMS320C5500"] } {

    set mempage 2

  }

  puts "Initializing SPI controller with base adress $ctrlr_base ..." 

  #SPIPRI = 0x10
  PutShort $DEV [expr $ctrlr_base + 15] $mempage 0x10
  puts [format "SPIPRI=0x%04x" [ReadShort $DEV [expr $ctrlr_base + 15] 1]] 

  # SPIFFTX = 0xe040
  PutShort $DEV [expr $ctrlr_base + 10] $mempage 0xe040
  puts [format "SPIFFTX=0x%04x" [ReadShort $DEV [expr $ctrlr_base + 10] 1]] 
  

  #SPIFFRX = 0x204f
  PutShort $DEV [expr $ctrlr_base + 11] $mempage 0x204f
  puts [format "SPIFFRX=0x%04x" [ReadShort $DEV [expr $ctrlr_base + 11] 1]] 

  #SPIFFCT = 0x0
  PutShort $DEV [expr $ctrlr_base + 12] $mempage 0x0000
  puts [format "SPIFFCT=0x%04x" [ReadShort $DEV [expr $ctrlr_base + 12] 1]] 

  #SPICCR = 0x0007
  PutShort $DEV [expr $ctrlr_base + 0]  $mempage 0x07
  puts [format "SPICCR=0x%04x" [ReadShort $DEV [expr $ctrlr_base + 0] 1]] 

  #SPICTL = 0x0006
  PutShort $DEV [expr $ctrlr_base + 1] $mempage 0x06
  puts [format "SPICTL=0x%04x" [ReadShort $DEV [expr $ctrlr_base + 1] 1]] 

  #SPIBRR = divisor
  PutShort $DEV [expr $ctrlr_base + 4] $mempage $divisor
  puts [format "SPIBRR=0x%04x" [ReadShort $DEV [expr $ctrlr_base + 4] 1]] 

  #SPICCR = 0x00c7
  PutShort $DEV [expr $ctrlr_base + 0] $mempage 0xC7
  puts [format "SPICCR=0x%04x" [ReadShort $DEV [expr $ctrlr_base + 0] 1]] 

  puts "Initializing SPI controller Finished"
}
