#

set TCLXDSPATH [file join $TCLXDSPATH]
set TCLXDSLIBFULL [file join $TCLXDSPATH $TCLXDSLIB]

proc ldpgm {coff dev} {

  set sections [xdsutil coff sections $coff]

  foreach section $sections {
    set flags [xdsutil coff sectionflags $coff $section]
    set loadable 0
    if { [lsearch -exact $flags "REG"] >= 0 } {
      set loadable 1
    } elseif { [lsearch -exact $flags "DATA"] >= 0 } {
      set loadable 1
    } elseif { [lsearch -exact $flags "TEXT"] >= 0 } {
      set loadable 1
    }
    if { [lsearch -exact $flags "COPY"] >= 0 ||
         [lsearch -exact $flags "DSECT"] >= 0 ||
         [lsearch -exact $flags "NOLOAD"] >= 0 } {
      set loadable 0
    }
    if { $loadable != 0 } {
      set addr [expr [lindex [split [lsearch -inline $flags "PHYSADDR*"] "="] 1]]
      set page [expr [lindex [split [lsearch -inline $flags "MEMPAGE*"] "="] 1]]
      set size [expr [lindex [split [lsearch -inline $flags "SIZE*"] "="] 1]]
      if { $size > 0 } {
        set hexad [format "%08x" $addr]
        puts -nonewline "  => $section @$page:$hexad $size bytes ..."
        set data [xdsutil coff sectiondata $coff $section]
        set data [xdsutil bintohexdata $data]
        xds status $dev
        set result [xds write_mem_coff $dev $addr $size $page $data]
        if { $result != 0 } {
          puts " failed"
          return $result
        }
        puts " ok"
      }
    }
  }
  return 0
}

proc xerror {dev err code} {

  puts "$err"
  puts [xds fetch_errors $dev]
  xds disconnect $dev
  xds quit $dev
  upvar #0 TCLXDSLIBFULL lib
  unload $lib
  exit $code

}

proc ferror {dev err code} {

  puts "$err"
  xds disconnect $dev
  xds quit $dev
  upvar #0 TCLXDSLIBFULL lib
  unload $lib
  exit $code

}

proc parmvalue { data name } {
  return [expr [lindex [split [lsearch -inline $data "$name*"] "="] 1]]
}

proc getsym { coff name } {

upvar 1 [format "%s_addr" $name] a
upvar 1 [format "%s_pg" $name] p

set sym [xdsutil coff symbol $coff $name ]
set a [parmvalue $sym VAL]
set p [parmvalue $sym MEMP]

}

proc PutShort { dev addr pg data } {

  upvar #0 TargetBigEndian t
  if { ($t == 1) || ($t == 2) } {
    set newdata [format %02x%02x [expr $data & 0xFF] [expr ( $data & 0xFF00 ) >> 8]]
  } else {
    set newdata [format %x $data]
  }

  set ec [xds write_mem $dev $addr 2 $pg $newdata]

  if { $ec != 0 } {
    xerror $dev "Error while transferring 'short int' number to target" 1
  }

}

proc PutLong { dev addr pg data } {

  upvar #0 TargetBigEndian t

  if { ($t == 1) || ($t == 2) } {
    set newdata [format %02x%02x%02x%02x \
                [expr   $data & 0xFF] \
                [expr ( $data & 0xFF00 ) >> 8 ] \
                [expr ( $data & 0xFF0000 ) >> 16 ] \
                [expr ( $data & 0xFF000000 ) >> 24 ]]
  } else {
    set newdata [format %x $data]
  }

  set ec [xds write_mem $dev $addr 4 $pg $newdata]

  if { $ec != 0 } {
    xerror $dev "Error while transferring 'long int' number to target" 1
  }

}

proc WaitForBP { dev timeout } {

  set ii 0

  while { 1 } {
    set stat [split [xds status $dev] " "]
    if { [lindex $stat 0] != 0 } {
      xerror $dev "Error getting target status" 1
    }
    set st [expr [lindex $stat 1]]
    if { ($st & 0x40) != 0 } {
      break
    }
    after 100
    incr ii
    if { $ii > $timeout } {
      xerror $dev "Timeout while waiting completion of target-side code" 1
    }
  }
}

proc ReadShort { dev addr pg } {

  upvar #0 TargetBigEndian t

  set result [split [xds read_mem $dev $addr 2 $pg]]

  if { [llength $result] == 1 } {
    xerror $dev "Error while transferring 'short int' number from target" 1
  }                 

  set result [lindex $result 1]
  set data [expr "0x$result"]

  if { ($t == 1) || ($t == 3) } {
    set data [expr ( ( $data & 0xFF ) << 8 ) | ( ( $data & 0xFF00 ) >> 8 )  ]
  } 

  return $data

}

proc ReadLong { dev addr pg } {

  upvar #0 TargetBigEndian t

  set result [split [xds read_mem $dev $addr 4 $pg]]

  if { [llength $result] == 1 } {
    xerror $dev "Error while transferring 'long int' number from target" 1
  }                 

  set result [lindex $result 1]
  set data [expr "0x$result"]

  if { ($t == 1) || ($t == 3) } {
    set data [expr ( ( ( $data & 0x000000FF ) << 24 ) | \
                     ( ( $data & 0x0000FF00 ) << 8  ) | \
                     ( ( $data & 0x00FF0000 ) >> 8  ) | \
                     ( ( $data & 0xFF000000 ) >> 24 ) ) ]
  } 

  return $data

}


proc ReadReg { dev reg } {

  set result [split [xds read_reg $dev $reg]]

  if { [llength $result] == 1 } {
    xerror $dev "Error while reading register $reg" 1
  }                 

  set result [lindex $result 1]
  set data [expr "0x$result"]

  return $data

}

proc WriteReg { dev reg data } {

  set newdata [format %x $data]

  set ec [xds write_reg $dev $newdata $reg]

  if { $ec != 0 } {
    xerror $dev "Error while writing $reg register" 1
  }

}

